@@ -5,7 +5,6 @@ import sys
 import stat
 import shutil
 import codecs
-from glob import glob
 from collections import defaultdict
 from cStringIO import StringIO
 from subprocess import Popen, PIPE
@@ -19,7 +18,8 @@ from django.template import TemplateSyntaxError
 register = template.Library()
 
 try:
-    from slimmer import css_slimmer, guessSyntax, html_slimmer, js_slimmer, xhtml_slimmer
+    from slimmer import (css_slimmer, guessSyntax, html_slimmer, js_slimmer,
+            xhtml_slimmer)
     slimmer = 'installed'
 except ImportError:
     slimmer = None
@@ -33,9 +33,11 @@ else:
     _CAN_SYMLINK = getattr(settings, "DJANGO_STATIC_USE_SYMLINK", True)
 
 DEBUG = settings.DEBUG
+settings.DJANGO_STATIC = getattr(settings, 'DJANGO_STATIC', False)
 DJANGO_STATIC_SAVE_PREFIX = getattr(settings, 'DJANGO_STATIC_SAVE_PREFIX', '')
 DJANGO_STATIC_NAME_PREFIX = getattr(settings, 'DJANGO_STATIC_NAME_PREFIX', '')
-MEDIA_URL = getattr(settings, "DJANGO_STATIC_MEDIA_URL", None)
+settings.DJANGO_STATIC_MEDIA_URL_ALWAYS = \
+  getattr(settings, "DJANGO_STATIC_MEDIA_URL_ALWAYS", False)
 MEDIA_ROOTS = getattr(settings, "DJANGO_STATIC_MEDIA_ROOTS",
         [ settings.MEDIA_ROOT ])
 
@@ -182,7 +184,9 @@ class StaticFileNode(template.Node):
 
     def render(self, context):
         filename = self.filename_var.resolve(context)
-        if not getattr(settings, 'DJANGO_STATIC', False):
+        if not settings.DJANGO_STATIC:
+            if settings.DJANGO_STATIC_MEDIA_URL_ALWAYS:
+                return settings.DJANGO_STATIC_MEDIA_URL + filename
             return filename
 
         new_filename = _static_file([x.strip() for x in filename.split(';')],
@@ -218,7 +222,8 @@ def do_staticallfiles(parser, token):
 
 SCRIPTS_REGEX = re.compile('<script [^>]*src=["\']([^"\']+)["\'].*?</script>')
 STYLES_REGEX = re.compile('<link.*?href=["\']([^"\']+)["\'].*?>', re.M|re.DOTALL)
-IMG_REGEX = re.compile('<img.*?src=["\']([^"\']+)["\'].*?>', re.M|re.DOTALL)
+IMG_REGEX = re.compile('<img.*?src=["\']((?!data:)[^"\']+)["\'].*?>', re.M|re.DOTALL)
+ASYNC_DEFER_REGEX = re.compile('async|defer')
 
 class StaticFilesNode(template.Node):
     """find all static files in the wrapped code and run staticfile (or
@@ -243,12 +248,28 @@ class StaticFilesNode(template.Node):
         which we already have routines for doing.
         """
         code = self.nodelist.render(context)
-        if not getattr(settings, 'DJANGO_STATIC', False):
+        if not settings.DJANGO_STATIC:
+            # Append MEDIA_URL if set
+            # quick and dirty
+            if settings.DJANGO_STATIC_MEDIA_URL_ALWAYS:
+                for match in STYLES_REGEX.finditer(code):
+                    for filename in match.groups():
+                        code = (code.replace(filename,
+                                             settings.DJANGO_STATIC_MEDIA_URL + filename))
+
+                for match in SCRIPTS_REGEX.finditer(code):
+                    for filename in match.groups():
+                        code = (code.replace(filename,
+                                             settings.DJANGO_STATIC_MEDIA_URL + filename))
+
+                return code
+
             return code
 
         new_js_filenames = []
         for match in SCRIPTS_REGEX.finditer(code):
             whole_tag = match.group()
+            async_defer = ASYNC_DEFER_REGEX.search(whole_tag)
             for filename in match.groups():
 
                 optimize_if_possible = self.optimize_if_possible
@@ -310,8 +331,11 @@ class StaticFilesNode(template.Node):
 
         if new_js_filename:
             # Now is the time to apply the name prefix if there is one
-            new_tag = '<script type="text/javascript" src="%s"></script>' % \
-              new_js_filename
+            if async_defer:
+                new_tag = ('<script %s src="%s"></script>' %
+                        (async_defer.group(0), new_js_filename))
+            else:
+                new_tag = '<script src="%s"></script>' % new_js_filename
             code = "%s%s" % (new_tag, code)
 
         for media_type, new_css_filename in new_css_filenames_combined.items():
@@ -321,7 +345,7 @@ class StaticFilesNode(template.Node):
 
         return code
 
-REFERRED_CSS_URLS_REGEX = re.compile('url\(([^\)]+)\)')
+REFERRED_CSS_URLS_REGEX = re.compile('url\(((?!data:)[^\)]+)\)')
 REFERRED_CSS_URLLESS_IMPORTS_REGEX = re.compile('@import\s+[\'"]([^\'"]+)[\'"]')
 
 def _static_file(filename,
@@ -330,12 +354,12 @@ def _static_file(filename,
                  warn_no_file=True):
     """
     """
-    if not getattr(settings, 'DJANGO_STATIC', False):
+    if not settings.DJANGO_STATIC:
         return file_proxy(filename, disabled=True)
 
     def wrap_up(filename):
-        if MEDIA_URL:
-            return MEDIA_URL + filename
+        if settings.DJANGO_STATIC_MEDIA_URL_ALWAYS:
+            return settings.DJANGO_STATIC_MEDIA_URL + filename
         return filename
 
     is_combined_files = isinstance(filename, list)
@@ -473,6 +497,7 @@ def _static_file(filename,
             # and _static_file() all images refered in the CSS file itself
             def replacer(match):
                 this_filename = match.groups()[0]
+
                 if (this_filename.startswith('"') and this_filename.endswith('"')) or \
                   (this_filename.startswith("'") and this_filename.endswith("'")):
                     this_filename = this_filename[1:-1]
@@ -517,7 +542,7 @@ def _static_file(filename,
             raise ValueError(
               "Unable to slimmer file %s. Unrecognized extension" % new_filename)
         #print "** STORING:", new_filepath
-        codecs.open(new_filepath, 'w', 'utf8').write(content)
+        codecs.open(new_filepath, 'w', 'utf-8').write(content)
     elif symlink_if_possible and not is_combined_files:
         #print "** SYMLINK:", filepath, '-->', new_filepath
 
